/*
 * Copyright 2009-2019 The VOTCA Development Team (http://www.votca.org)
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 *
 */

#ifndef VOTCA_CSG_ANALYSISTOOL_H
#define VOTCA_CSG_ANALYSISTOOL_H

#include "../../include/votca/csg/cgengine.h"
#include "bondedstatistics.h"
#include <map>
#include <string>

/**
    \brief base class for all analysis tools

    This is the base class for all analysis tool.
    \todo do option functions!!!
*/
namespace votca {
namespace csg {

class AnalysisTool {
 public:
  AnalysisTool() = default;
  virtual ~AnalysisTool() = default;

  virtual void Register(std::map<std::string, AnalysisTool *> &lib) = 0;
  virtual void Command(BondedStatistics &bs, const std::string &cmd,
                       std::vector<std::string> &args) = 0;
  virtual void Help(const std::string &cmd, std::vector<std::string> &args) = 0;
};

}  // namespace csg
}  // namespace votca
#endif  // VOTCA_CSG_ANALYSISTOOL_H
