/*
 * Copyright 2009-2019 The VOTCA Development Team (http://www.votca.org)
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 *
 */

#ifndef VOTCA_CSG_STDANALYSIS_H
#define VOTCA_CSG_STDANALYSIS_H

#include "analysistool.h"
#include "bondedstatistics.h"
#include <map>

namespace votca {
namespace csg {

class StdAnalysis : public AnalysisTool {
 public:
  StdAnalysis() = default;
  ~StdAnalysis() override = default;

  void Register(std::map<std::string, AnalysisTool *> &lib) override;

  void Command(BondedStatistics &bs, const std::string &cmd,
               std::vector<std::string> &args) override;
  void Help(const std::string &cmd, std::vector<std::string> &args) override;

  void WriteValues(BondedStatistics &bs, std::vector<std::string> &args);
  void WriteCorrelations(BondedStatistics &bs, std::vector<std::string> &args);
  void WriteAutocorrelation(BondedStatistics &bs,
                            std::vector<std::string> &args);
};

}  // namespace csg
}  // namespace votca
#endif  // VOTCA_CSG_STDANALYSIS_H
